@@ -30,10 +30,10 @@ from numbers import Number
 from pprint import pformat
 from typing import Any
 from typing import Dict
-from typing import final
 from typing import Optional
 from typing import Tuple
 from typing import Union
+from typing import final
 
 import torch
 
@@ -42,7 +42,6 @@ from disent.nn.modules import DisentLightningModule
 from disent.schedule import Schedule
 from disent.util.imports import import_obj
 
-
 log = logging.getLogger(__name__)
 
 
@@ -52,7 +51,6 @@ log = logging.getLogger(__name__)
 
 
 class DisentConfigurable(object):
-
     @dataclass
     class cfg(object):
         def get_keys(self) -> list:
@@ -67,9 +65,9 @@ class DisentConfigurable(object):
     def __init__(self, cfg: cfg = cfg()):
         if cfg is None:
             cfg = self.__class__.cfg()
-            log.info(f'Initialised default config {cfg=} for {self.__class__.__name__}')
+            log.info(f"Initialised default config {cfg=} for {self.__class__.__name__}")
         super().__init__()
-        assert isinstance(cfg, self.__class__.cfg), f'{cfg=} ({type(cfg)}) is not an instance of {self.__class__.cfg}'
+        assert isinstance(cfg, self.__class__.cfg), f"{cfg=} ({type(cfg)}) is not an instance of {self.__class__.cfg}"
         self.cfg = cfg
 
 
@@ -79,11 +77,12 @@ class DisentConfigurable(object):
 
 
 class DisentFramework(DisentConfigurable, DisentLightningModule):
-
     @dataclass
     class cfg(DisentConfigurable.cfg):
         # optimizer config
-        optimizer: Union[str] = 'adam'  # name in the registry, eg. `adam` OR the path to an optimizer eg. `torch.optim.Adam`
+        optimizer: Union[
+            str
+        ] = "adam"  # name in the registry, eg. `adam` OR the path to an optimizer eg. `torch.optim.Adam`
         optimizer_kwargs: Optional[Dict[str, Union[str, float, int]]] = None
 
     def __init__(
@@ -99,7 +98,9 @@ class DisentFramework(DisentConfigurable, DisentLightningModule):
         self.cfg.optimizer_kwargs = self._check_optimizer_kwargs(self.cfg.optimizer_kwargs)
         # batch augmentations may not be implemented as dataset
         # transforms so we can apply these on the GPU instead
-        assert callable(batch_augment) or (batch_augment is None), f'invalid batch_augment: {repr(batch_augment)}, must be callable or `None`'
+        assert callable(batch_augment) or (
+            batch_augment is None
+        ), f"invalid batch_augment: {repr(batch_augment)}, must be callable or `None`"
         self._batch_augment = batch_augment
         # schedules
         # - maybe add support for schedules in the config?
@@ -109,27 +110,31 @@ class DisentFramework(DisentConfigurable, DisentLightningModule):
     @staticmethod
     def _check_optimizer(optimizer: str):
         if not isinstance(optimizer, str):
-            raise TypeError(f'invalid optimizer: {repr(optimizer)}, must be a `str`')
+            raise TypeError(f"invalid optimizer: {repr(optimizer)}, must be a `str`")
         # check that the optimizer has been registered
         # otherwise check that the optimizer class can be imported instead
         if optimizer not in registry.OPTIMIZERS:
             try:
                 import_obj(optimizer)
             except ImportError:
-                raise KeyError(f'invalid optimizer: {repr(optimizer)}, valid optimizers are: {sorted(registry.OPTIMIZERS)}, or an import path to an optimizer, eg. `torch.optim.Adam`')
+                raise KeyError(
+                    f"invalid optimizer: {repr(optimizer)}, valid optimizers are: {sorted(registry.OPTIMIZERS)}, or an import path to an optimizer, eg. `torch.optim.Adam`"
+                )
         # return the updated values!
         return optimizer
 
     @staticmethod
     def _check_optimizer_kwargs(optimizer_kwargs: Optional[dict]):
         # check the optimizer kwargs
-        assert isinstance(optimizer_kwargs, dict) or (optimizer_kwargs is None), f'invalid optimizer_kwargs type, got: {type(optimizer_kwargs)}'
+        assert isinstance(optimizer_kwargs, dict) or (
+            optimizer_kwargs is None
+        ), f"invalid optimizer_kwargs type, got: {type(optimizer_kwargs)}"
         # get default kwargs OR copy
         optimizer_kwargs = dict() if (optimizer_kwargs is None) else dict(optimizer_kwargs)
         # set default values
-        if 'lr' not in optimizer_kwargs:
-            optimizer_kwargs['lr'] = 1e-3
-            log.info('lr not specified in `optimizer_kwargs`, setting to default value of `1e-3`')
+        if "lr" not in optimizer_kwargs:
+            optimizer_kwargs["lr"] = 1e-3
+            log.info("lr not specified in `optimizer_kwargs`, setting to default value of `1e-3`")
         # return the updated values
         return optimizer_kwargs
 
@@ -144,12 +149,14 @@ class DisentFramework(DisentConfigurable, DisentLightningModule):
             optimizer_cls = import_obj(self.cfg.optimizer)
         # check that we can call the optimizer
         if not callable(optimizer_cls):
-            raise TypeError(f'unsupported optimizer type: {type(optimizer_cls)}')
+            raise TypeError(f"unsupported optimizer type: {type(optimizer_cls)}")
         # instantiate class
         optimizer_instance = optimizer_cls(self.parameters(), **self.cfg.optimizer_kwargs)
         # check instance
         if not isinstance(optimizer_instance, torch.optim.Optimizer):
-            raise TypeError(f'returned object is not an instance of torch.optim.Optimizer, got: {type(optimizer_instance)}')
+            raise TypeError(
+                f"returned object is not an instance of torch.optim.Optimizer, got: {type(optimizer_instance)}"
+            )
         # return the optimizer
         return optimizer_instance
 
@@ -170,7 +177,7 @@ class DisentFramework(DisentConfigurable, DisentLightningModule):
             loss = self.do_training_step(batch, batch_idx)
             # check loss values
             self._assert_valid_loss(loss)
-            self.log('loss', float(loss), prog_bar=True)
+            self.log("loss", float(loss), prog_bar=True)
             # return loss
             return loss
         except Exception as e:  # pragma: no cover
@@ -203,9 +210,9 @@ class DisentFramework(DisentConfigurable, DisentLightningModule):
     @final
     def _assert_valid_loss(self, loss):
         if torch.isnan(loss) or torch.isinf(loss):
-            raise ValueError('The returned loss is nan or inf')
-        if loss > 1e+20:
-            raise ValueError(f'The returned loss: {loss:.2e} is out of bounds: > {1e+20:.0e}')
+            raise ValueError("The returned loss is nan or inf")
+        if loss > 1e20:
+            raise ValueError(f"The returned loss: {loss:.2e} is out of bounds: > {1e+20:.0e}")
 
     def forward(self, batch) -> torch.Tensor:  # pragma: no cover
         """this function should return the single final output of the model, including the final activation"""
@@ -232,7 +239,7 @@ class DisentFramework(DisentConfigurable, DisentLightningModule):
             self._registered_schedules.remove(target)
             self._active_schedules.pop(target)
         else:
-            raise KeyError(f'Cannot remove schedule for target {repr(target)} that does not exist!')
+            raise KeyError(f"Cannot remove schedule for target {repr(target)} that does not exist!")
 
     @final
     def register_schedule(self, target: str, schedule: Schedule, logging=True) -> bool:
@@ -245,7 +252,7 @@ class DisentFramework(DisentConfigurable, DisentLightningModule):
         assert type(schedule) is not Schedule
         # handle the case where a schedule for the target already exists!
         if self.has_schedule(target):
-            raise KeyError(f'A schedule for target {repr(target)} has already been registered!')
+            raise KeyError(f"A schedule for target {repr(target)} has already been registered!")
         # check the target exists!
         possible_targets = {f.name for f in fields(self.cfg)}
         # register this target
@@ -255,11 +262,13 @@ class DisentFramework(DisentConfigurable, DisentLightningModule):
             initial_val = getattr(self.cfg, target)
             self._active_schedules[target] = (initial_val, schedule)
             if logging:
-                log.info(f'Activating schedule for target {repr(target)} on {repr(self.__class__.__name__)}.')
+                log.info(f"Activating schedule for target {repr(target)} on {repr(self.__class__.__name__)}.")
             return True
         else:
             if logging:
-                log.warning(f'Unactivated schedule for target {repr(target)} on {repr(self.__class__.__name__)} because the key was not found in the config.')
+                log.warning(
+                    f"Unactivated schedule for target {repr(target)} on {repr(self.__class__.__name__)} because the key was not found in the config."
+                )
             return False
 
     @final
@@ -267,7 +276,7 @@ class DisentFramework(DisentConfigurable, DisentLightningModule):
         if not self._active_schedules:
             return
         # log the step value
-        self.log('scheduler/step', self.trainer.global_step)
+        self.log("scheduler/step", self.trainer.global_step)
         # update the values
         for target, (initial_val, scheduler) in self._active_schedules.items():
             # get the scheduled value
@@ -275,7 +284,7 @@ class DisentFramework(DisentConfigurable, DisentLightningModule):
             # update it on the config
             setattr(self.cfg, target, new_value)
             # log that things changed
-            self.log(f'scheduled/{target}', new_value)
+            self.log(f"scheduled/{target}", new_value)
 
 
 # ========================================================================= #
