@@ -24,19 +24,18 @@
 
 import logging
 from dataclasses import dataclass
-from typing import final
 from typing import Optional
 from typing import Sequence
+from typing import final
 
 import torch
 from torch.distributions import Normal
 
-from disent.frameworks.helper.reconstructions import make_reconstruction_loss
 from disent.frameworks.helper.reconstructions import ReconLossHandler
+from disent.frameworks.helper.reconstructions import make_reconstruction_loss
 from disent.frameworks.vae._supervised__adaneg_tvae import AdaNegTripletVae
 from disent.nn.loss.triplet_mining import configured_idx_mine
 
-
 log = logging.getLogger(__name__)
 
 
@@ -46,19 +45,18 @@ log = logging.getLogger(__name__)
 
 
 class DataOverlapMixin(object):
-
     # should be inherited by the config on the child class
     @dataclass
     class cfg:
         # override from AE
-        recon_loss: str = 'mse'
+        recon_loss: str = "mse"
         # OVERLAP VAE
         overlap_loss: Optional[str] = None  # if None, use the value from recon_loss
         overlap_num: int = 1024
         overlap_mine_ratio: float = 0.1
-        overlap_mine_triplet_mode: str = 'none'
+        overlap_mine_triplet_mode: str = "none"
         # AUGMENT
-        overlap_augment_mode: str = 'augment'
+        overlap_augment_mode: str = "augment"
         overlap_augment: Optional[dict] = None
 
     # private properties
@@ -70,24 +68,31 @@ class DataOverlapMixin(object):
     _init: bool
 
     def init_data_overlap_mixin(self):
-        if hasattr(self, '_init'):
-            raise RuntimeError(f'{DataOverlapMixin.__name__} on {self.__class__.__name__} was initialised more than once!')
+        if hasattr(self, "_init"):
+            raise RuntimeError(
+                f"{DataOverlapMixin.__name__} on {self.__class__.__name__} was initialised more than once!"
+            )
         self._init = True
         # set augment and instantiate if needed
         if self.cfg.overlap_augment is not None:
             import hydra
+
             self._augment = hydra.utils.instantiate(self.cfg.overlap_augment)
-            assert (self._augment is None) or callable(self._augment), f'augment is not None or callable: {repr(self._augment)}, obtained from `overlap_augment={repr(self.cfg.overlap_augment)}`'
+            assert (self._augment is None) or callable(
+                self._augment
+            ), f"augment is not None or callable: {repr(self._augment)}, obtained from `overlap_augment={repr(self.cfg.overlap_augment)}`"
         else:
             self._augment = None
         # get overlap loss
         if self.cfg.overlap_loss is None:
-            log.info(f'`overlap_loss` not specified for {repr(self.__class__.__name__)}, using `recon_loss` instead: {repr(self.cfg.recon_loss)}')
+            log.info(
+                f"`overlap_loss` not specified for {repr(self.__class__.__name__)}, using `recon_loss` instead: {repr(self.cfg.recon_loss)}"
+            )
             overlap_loss = self.cfg.recon_loss
         else:
             overlap_loss = self.cfg.overlap_loss
         # construct the overlap handler
-        self._overlap_handler: ReconLossHandler = make_reconstruction_loss(name=overlap_loss, reduction='mean')
+        self._overlap_handler: ReconLossHandler = make_reconstruction_loss(name=overlap_loss, reduction="mean")
 
     @final
     @property
@@ -115,7 +120,7 @@ class DataOverlapMixin(object):
         with torch.no_grad():
             a_p_losses = self.overlap_handler.compute_pairwise_loss(a_x_targ_OLD, p_x_targ_OLD)  # (B, C, H, W) -> (B,)
             a_n_losses = self.overlap_handler.compute_pairwise_loss(a_x_targ_OLD, n_x_targ_OLD)  # (B, C, H, W) -> (B,)
-            swap_mask = (a_p_losses > a_n_losses)  # (B,)
+            swap_mask = a_p_losses > a_n_losses  # (B,)
         # ++++++++++++++++++++++++++++++++++++++++++ #
         return swap_mask
 
@@ -123,18 +128,19 @@ class DataOverlapMixin(object):
     def augment_batch(self, x_targ):
         # ++++++++++++++++++++++++++++++++++++++++++ #
         # perform the augments
-        if self.cfg.overlap_augment_mode in ('augment', 'augment_each'):
+        if self.cfg.overlap_augment_mode in ("augment", "augment_each"):
             # recreate augment each time
-            if self.cfg.overlap_augment_mode == 'augment_each':
+            if self.cfg.overlap_augment_mode == "augment_each":
                 import hydra
+
                 self._augment = hydra.utils.instantiate(self.cfg.overlap_augment)
             # augment on correct device, but skip if not defined!
             aug_x_targ = x_targ if (self._augment is None) else self._augment(x_targ)
-        elif self.cfg.overlap_augment_mode == 'none':
+        elif self.cfg.overlap_augment_mode == "none":
             # no augment
             aug_x_targ = x_targ
         else:
-            raise KeyError(f'invalid cfg.overlap_augment_mode={repr(self.cfg.overlap_augment_mode)}')
+            raise KeyError(f"invalid cfg.overlap_augment_mode={repr(self.cfg.overlap_augment_mode)}")
         # ++++++++++++++++++++++++++++++++++++++++++ #
         # checks
         assert x_targ.shape == aug_x_targ.shape
@@ -155,7 +161,9 @@ class DataOverlapMixin(object):
         # 1. augment batch
         aug_x_targ = self.augment_batch(x_targ_orig)
         # 2. generate random triples -- this does not generate unique pairs
-        a_idxs, p_idxs, n_idxs = torch.randint(len(aug_x_targ), size=(3, min(self.cfg.overlap_num, len(aug_x_targ)**3)), device=aug_x_targ.device)
+        a_idxs, p_idxs, n_idxs = torch.randint(
+            len(aug_x_targ), size=(3, min(self.cfg.overlap_num, len(aug_x_targ) ** 3)), device=aug_x_targ.device
+        )
         # ++++++++++++++++++++++++++++++++++++++++++ #
         # self.debug(x_targ_orig, x_targ, a_idxs, p_idxs, n_idxs)
         # ++++++++++++++++++++++++++++++++++++++++++ #
@@ -214,12 +222,14 @@ class DataOverlapTripletVae(AdaNegTripletVae, DataOverlapMixin):
     class cfg(AdaNegTripletVae.cfg, DataOverlapMixin.cfg):
         pass
 
-    def __init__(self, model: 'AutoEncoder', cfg: cfg = None, batch_augment=None):
+    def __init__(self, model: "AutoEncoder", cfg: cfg = None, batch_augment=None):
         super().__init__(model=model, cfg=cfg, batch_augment=batch_augment)
         # initialise mixin
         self.init_data_overlap_mixin()
 
-    def hook_compute_ave_aug_loss(self, ds_posterior: Sequence[Normal], ds_prior, zs_sampled, xs_partial_recon, xs_targ: Sequence[torch.Tensor]):
+    def hook_compute_ave_aug_loss(
+        self, ds_posterior: Sequence[Normal], ds_prior, zs_sampled, xs_partial_recon, xs_targ: Sequence[torch.Tensor]
+    ):
         [d_posterior], [x_targ_orig] = ds_posterior, xs_targ
         # 1. randomly generate and mine triplets using augmented versions of the inputs
         a_idxs, p_idxs, n_idxs = self.random_mined_triplets(x_targ_orig=x_targ_orig)
