@@ -38,20 +38,19 @@ from typing import Union
 import numpy as np
 import torch
 import torch.nn.functional as F
-from disent.frameworks.helper.reconstructions import ReconLossHandler
 from scipy.stats import pearsonr
 from scipy.stats import spearmanr
 
 from disent.dataset import DisentDataset
+from disent.frameworks.helper.reconstructions import ReconLossHandler
+from disent.metrics._flatness import encode_all_along_factor
+from disent.metrics._flatness import encode_all_factors
+from disent.metrics._flatness import filter_inactive_factors
 from disent.metrics.utils import make_metric
 from disent.nn.functional import torch_mean_generalized
 from disent.nn.functional import torch_pca
 from disent.nn.loss.reduction import batch_loss_reduction
 from disent.util import to_numpy
-from disent.metrics._flatness import encode_all_along_factor
-from disent.metrics._flatness import encode_all_factors
-from disent.metrics._flatness import filter_inactive_factors
-
 
 log = logging.getLogger(__name__)
 
@@ -71,14 +70,14 @@ _SAMPLES_MULTIPLIER_FACTOR = 2
 
 
 def _metric_factored_components(
-        dataset: DisentDataset,
-        representation_function: callable,
-        num_samples: int = 64,
-        global_subset_size: int = 32,
-        repeats: int = 1024,
-        batch_size: int = 64,
-        compute_distances: bool = True,
-        compute_linearity: bool = True,
+    dataset: DisentDataset,
+    representation_function: callable,
+    num_samples: int = 64,
+    global_subset_size: int = 32,
+    repeats: int = 1024,
+    batch_size: int = 64,
+    compute_distances: bool = True,
+    compute_linearity: bool = True,
 ):
     """
     Michlo et al.
@@ -133,7 +132,9 @@ def _metric_factored_components(
     """
     # checks
     if not (compute_distances or compute_linearity):
-        raise ValueError(f'Metric will not compute any values! At least one of: `compute_distances` or `compute_linearity` must be `True`')
+        raise ValueError(
+            f"Metric will not compute any values! At least one of: `compute_distances` or `compute_linearity` must be `True`"
+        )
 
     # compute actual metric values
     factor_scores, global_scores = _compute_factored_metric_components(
@@ -145,7 +146,6 @@ def _metric_factored_components(
         batch_size=batch_size,
         compute_distances=compute_distances,
         compute_linearity=compute_linearity,
-
     )
 
     # convert values from torch
@@ -160,21 +160,21 @@ def _metric_factored_components(
 
 # EXPORT: metric_factored_components
 metric_factored_components = make_metric(
-    'factored_components',
+    "factored_components",
     default_kwargs=dict(compute_distances=True, compute_linearity=True),
     fast_kwargs=dict(compute_distances=True, compute_linearity=True, repeats=128),
 )(_metric_factored_components)
 
 # EXPORT: metric_distances
 metric_distances = make_metric(
-    'distances',
+    "distances",
     default_kwargs=dict(compute_distances=True, compute_linearity=False),
     fast_kwargs=dict(compute_distances=True, compute_linearity=False, repeats=128),
 )(_metric_factored_components)
 
 # EXPORT: metric_linearity
 metric_linearity = make_metric(
-    'linearity',
+    "linearity",
     default_kwargs=dict(compute_distances=False, compute_linearity=True),
     fast_kwargs=dict(compute_distances=False, compute_linearity=True, repeats=128),
 )(_metric_factored_components)
@@ -201,39 +201,40 @@ def _filtered_mean(values: torch.Tensor, p: Union[str, int], factor_sizes: Tuple
 
 @torch.no_grad()
 def _compute_factored_metric_components(
-        dataset: DisentDataset,
-        representation_function,
-        num_samples: int,
-        global_subset_size: int,
-        repeats: int,
-        batch_size: int,
-        compute_distances: bool,
-        compute_linearity: bool,
+    dataset: DisentDataset,
+    representation_function,
+    num_samples: int,
+    global_subset_size: int,
+    repeats: int,
+    batch_size: int,
+    compute_distances: bool,
+    compute_linearity: bool,
 ) -> (dict, dict):
-
     # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~- #
     # COMPUTE FOR EACH FACTOR
     # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~- #
 
     # shapes: (num_factors,)
-    factor_values: Dict[str, np.ndarray] = _numpy_stack_all_dicts([
-        _compute_factored_metric_components_along_factor(
-            dataset,
-            representation_function,
-            f_idx=f_idx,
-            num_samples=num_samples,
-            repeats=repeats,
-            batch_size=batch_size,
-            compute_distances=compute_distances,
-            compute_linearity=compute_linearity,
-        )
-        for f_idx in range(dataset.gt_data.num_factors)
-    ])
+    factor_values: Dict[str, np.ndarray] = _numpy_stack_all_dicts(
+        [
+            _compute_factored_metric_components_along_factor(
+                dataset,
+                representation_function,
+                f_idx=f_idx,
+                num_samples=num_samples,
+                repeats=repeats,
+                batch_size=batch_size,
+                compute_distances=compute_distances,
+                compute_linearity=compute_linearity,
+            )
+            for f_idx in range(dataset.gt_data.num_factors)
+        ]
+    )
 
     # aggregate for each factor
     # -- filter out factor dimensions that are incorrect. ie. size <= 1
     factor_scores = {
-        k: float(_filtered_mean(torch.from_numpy(v), p='geometric', factor_sizes=dataset.gt_data.factor_sizes))
+        k: float(_filtered_mean(torch.from_numpy(v), p="geometric", factor_sizes=dataset.gt_data.factor_sizes))
         for k, v in factor_values.items()
     }
 
@@ -250,7 +251,9 @@ def _compute_factored_metric_components(
             # sample random factors
             factors = dataset.gt_data.sample_factors(size=global_subset_size)
             # encode factors
-            zs, xs = encode_all_factors(dataset, representation_function, factors, batch_size=batch_size, return_batch=True)
+            zs, xs = encode_all_factors(
+                dataset, representation_function, factors, batch_size=batch_size, return_batch=True
+            )
             zs, xs, factors = zs.cpu(), xs.cpu(), torch.from_numpy(factors).to(torch.float32)
             # [COMPUTE SAME RATIO & CORRELATION]: was `_SAMPLES_MULTIPLIER_GLOBAL*len(zs)`
             computed_dists = _compute_dists(num_triplets=num_samples, zs_traversal=zs, xs_traversal=xs, factors=factors)
@@ -261,8 +264,8 @@ def _compute_factored_metric_components(
         # concatenate all into arrays: <shape: (repeats*num_samples,)>
         # then aggregate over first dimension: <shape: (,)>
         distance_measures: Dict[str, np.ndarray] = _numpy_concat_all_dicts(distance_measures)
-        distance_measures: Dict[str, float]      = _compute_scores_from_dists(distance_measures)
-        distance_measures: Dict[str, float]      = {f'distances.{k}.global': v for k, v in distance_measures.items()}
+        distance_measures: Dict[str, float] = _compute_scores_from_dists(distance_measures)
+        distance_measures: Dict[str, float] = {f"distances.{k}.global": v for k, v in distance_measures.items()}
     else:
         distance_measures: Dict[str, float] = {}
 
@@ -297,7 +300,7 @@ def _compute_unsorted_linear_values(zs_traversal, use_std: bool = True):
     # correlation along arbitrary orthogonal basis
     # -- note pca_mode='svd' returns the number of values equal to: min(factor_size, z_size)  !!! this may lower scores on average
     # -- note pca_mode='eig' returns the number of values equal to: z_size
-    _, linear_values = torch_pca(zs_traversal, center=True, mode='eig')
+    _, linear_values = torch_pca(zs_traversal, center=True, mode="eig")
     if use_std:
         linear_values = torch.sqrt(linear_values)
     return linear_values
@@ -316,7 +319,7 @@ def _score_from_sorted(sorted_vars: torch.Tensor, top_2: bool = False, norm: boo
         # for: x/(x+a)
         # normalised = (x/(x+a) - (1/n)) / (1 - (1/n))
         # normalised = (x - 1/(n-1) * a) / (x + a)
-        r = (r - (1/n)) / (1 - (1/n))
+        r = (r - (1 / n)) / (1 - (1 / n))
     # done!
     return r
 
@@ -382,11 +385,17 @@ def _unswapped_ratio_numpy(ap0: np.ndarray, an0: np.ndarray, ap1: np.ndarray, an
     # values must correspond
     same_mask = _same_mask(ap0=ap0, an0=an0, ap1=ap1, an1=an1)
     # num values
-    return np.mean(same_mask, dtype='float32')
+    return np.mean(same_mask, dtype="float32")
 
 
-def _compute_dists(num_triplets: int, zs_traversal: Optional[torch.Tensor], xs_traversal: torch.Tensor, factors: Optional[torch.Tensor], recon_loss_fn=F.mse_loss) -> Dict[str, np.ndarray]:
-    assert (factors      is None) or (len(factors)      == len(xs_traversal))
+def _compute_dists(
+    num_triplets: int,
+    zs_traversal: Optional[torch.Tensor],
+    xs_traversal: torch.Tensor,
+    factors: Optional[torch.Tensor],
+    recon_loss_fn=F.mse_loss,
+) -> Dict[str, np.ndarray]:
+    assert (factors is None) or (len(factors) == len(xs_traversal))
     assert (zs_traversal is None) or (len(zs_traversal) == len(xs_traversal))
 
     # get the recon loss function
@@ -394,7 +403,7 @@ def _compute_dists(num_triplets: int, zs_traversal: Optional[torch.Tensor], xs_t
         if isinstance(recon_loss_fn, ReconLossHandler):
             return recon_loss_fn.compute_unreduced_loss(input, target)
         else:
-            return recon_loss_fn(input, target, reduction='none')
+            return recon_loss_fn(input, target, reduction="none")
 
     # compute!
     with torch.no_grad():
@@ -404,19 +413,53 @@ def _compute_dists(num_triplets: int, zs_traversal: Optional[torch.Tensor], xs_t
         idxs_a, idxs_p, idxs_n = torch.randint(0, len(xs_traversal), size=(3, num_triplets), device=xs_traversal.device)
         # compute distances -- shape: (num,)
         distances = {
-            'ap_ground_dists': (torch.norm(factors[idxs_a, :] - factors[idxs_p, :], p=1, dim=-1) if (factors is not None) else torch.abs(idxs_a - idxs_p)).cpu().numpy(),
-            'an_ground_dists': (torch.norm(factors[idxs_a, :] - factors[idxs_n, :], p=1, dim=-1) if (factors is not None) else torch.abs(idxs_a - idxs_n)).cpu().numpy(),
-            'ap_data_dists':   batch_loss_reduction(_unreduced_loss(xs_traversal[idxs_a, ...], xs_traversal[idxs_p, ...]), reduction_dtype=torch.float32, reduction='mean').cpu().numpy(),
-            'an_data_dists':   batch_loss_reduction(_unreduced_loss(xs_traversal[idxs_a, ...], xs_traversal[idxs_n, ...]), reduction_dtype=torch.float32, reduction='mean').cpu().numpy(),
+            "ap_ground_dists": (
+                torch.norm(factors[idxs_a, :] - factors[idxs_p, :], p=1, dim=-1)
+                if (factors is not None)
+                else torch.abs(idxs_a - idxs_p)
+            )
+            .cpu()
+            .numpy(),
+            "an_ground_dists": (
+                torch.norm(factors[idxs_a, :] - factors[idxs_n, :], p=1, dim=-1)
+                if (factors is not None)
+                else torch.abs(idxs_a - idxs_n)
+            )
+            .cpu()
+            .numpy(),
+            "ap_data_dists": batch_loss_reduction(
+                _unreduced_loss(xs_traversal[idxs_a, ...], xs_traversal[idxs_p, ...]),
+                reduction_dtype=torch.float32,
+                reduction="mean",
+            )
+            .cpu()
+            .numpy(),
+            "an_data_dists": batch_loss_reduction(
+                _unreduced_loss(xs_traversal[idxs_a, ...], xs_traversal[idxs_n, ...]),
+                reduction_dtype=torch.float32,
+                reduction="mean",
+            )
+            .cpu()
+            .numpy(),
         }
         # compute distances -- shape: (num,)
         if zs_traversal is not None:
-            distances.update({
-                'ap_latent_dists.l1': torch.norm(zs_traversal[idxs_a, :] - zs_traversal[idxs_p, :], dim=-1, p=1).cpu().numpy(),
-                'an_latent_dists.l1': torch.norm(zs_traversal[idxs_a, :] - zs_traversal[idxs_n, :], dim=-1, p=1).cpu().numpy(),
-                'ap_latent_dists.l2': torch.norm(zs_traversal[idxs_a, :] - zs_traversal[idxs_p, :], dim=-1, p=2).cpu().numpy(),
-                'an_latent_dists.l2': torch.norm(zs_traversal[idxs_a, :] - zs_traversal[idxs_n, :], dim=-1, p=2).cpu().numpy(),
-            })
+            distances.update(
+                {
+                    "ap_latent_dists.l1": torch.norm(zs_traversal[idxs_a, :] - zs_traversal[idxs_p, :], dim=-1, p=1)
+                    .cpu()
+                    .numpy(),
+                    "an_latent_dists.l1": torch.norm(zs_traversal[idxs_a, :] - zs_traversal[idxs_n, :], dim=-1, p=1)
+                    .cpu()
+                    .numpy(),
+                    "ap_latent_dists.l2": torch.norm(zs_traversal[idxs_a, :] - zs_traversal[idxs_p, :], dim=-1, p=2)
+                    .cpu()
+                    .numpy(),
+                    "an_latent_dists.l2": torch.norm(zs_traversal[idxs_a, :] - zs_traversal[idxs_n, :], dim=-1, p=2)
+                    .cpu()
+                    .numpy(),
+                }
+            )
         # return values -- shape: (num,)
         return distances
 
@@ -424,54 +467,66 @@ def _compute_dists(num_triplets: int, zs_traversal: Optional[torch.Tensor], xs_t
 def _compute_scores_from_dists(dists: Dict[str, np.array]) -> Dict[str, float]:
     # [DATA & GROUND DISTS]:
     # extract the distances -- shape: (num,)
-    ap_ground_dists    = dists['ap_ground_dists']
-    an_ground_dists    = dists['an_ground_dists']
-    ap_data_dists      = dists['ap_data_dists']
-    an_data_dists      = dists['an_data_dists']
+    ap_ground_dists = dists["ap_ground_dists"]
+    an_ground_dists = dists["an_ground_dists"]
+    ap_data_dists = dists["ap_data_dists"]
+    an_data_dists = dists["an_data_dists"]
     # concatenate values -- shape: (2 * num,)
-    ground_dists    = np.concatenate([ap_ground_dists,    an_ground_dists],    axis=0)
-    data_dists      = np.concatenate([ap_data_dists,      an_data_dists],      axis=0)
+    ground_dists = np.concatenate([ap_ground_dists, an_ground_dists], axis=0)
+    data_dists = np.concatenate([ap_data_dists, an_data_dists], axis=0)
     # compute the scores
     # - check the number of swapped elements along a factor for random triplets.
     # - compute the spearman rank correlation coefficient over the concatenated distances
     # - compute the pearman correlation coefficient over the concatenated distances
     scores = {
-        'rsame_ground_data': _unswapped_ratio_numpy(ap0=ap_ground_dists, an0=an_ground_dists, ap1=ap_data_dists, an1=an_data_dists), # simplifies to: (ap_data_dists > an_data_dists).to(torch.float32).mean()
-        'rcorr_ground_data': spearmanr(ground_dists, data_dists)[0],
-        'lcorr_ground_data': pearsonr(ground_dists, data_dists)[0],
+        "rsame_ground_data": _unswapped_ratio_numpy(
+            ap0=ap_ground_dists, an0=an_ground_dists, ap1=ap_data_dists, an1=an_data_dists
+        ),  # simplifies to: (ap_data_dists > an_data_dists).to(torch.float32).mean()
+        "rcorr_ground_data": spearmanr(ground_dists, data_dists)[0],
+        "lcorr_ground_data": pearsonr(ground_dists, data_dists)[0],
     }
 
     # [RETURN EARLY]:
-    if 'ap_latent_dists.l1' not in dists:
+    if "ap_latent_dists.l1" not in dists:
         return scores
 
     # [LATENT DISTS]:
     # extract the distances -- shape: (num,)
-    ap_latent_dists_l1 = dists['ap_latent_dists.l1']
-    an_latent_dists_l1 = dists['an_latent_dists.l1']
-    ap_latent_dists_l2 = dists['ap_latent_dists.l2']
-    an_latent_dists_l2 = dists['an_latent_dists.l2']
+    ap_latent_dists_l1 = dists["ap_latent_dists.l1"]
+    an_latent_dists_l1 = dists["an_latent_dists.l1"]
+    ap_latent_dists_l2 = dists["ap_latent_dists.l2"]
+    an_latent_dists_l2 = dists["an_latent_dists.l2"]
     # concatenate values -- shape: (2 * num,)
     latent_dists_l1 = np.concatenate([ap_latent_dists_l1, an_latent_dists_l1], axis=0)
     latent_dists_l2 = np.concatenate([ap_latent_dists_l2, an_latent_dists_l2], axis=0)
     # compute the scores
-    scores.update({
-        # - check the number of swapped elements along a factor for random triplets.
-        'rsame_ground_latent.l1': _unswapped_ratio_numpy(ap0=ap_ground_dists,    an0=an_ground_dists,    ap1=ap_latent_dists_l1, an1=an_latent_dists_l1),  # simplifies to: (ap_latent_dists > an_latent_dists).to(torch.float32).mean()
-        'rsame_latent_data.l1':   _unswapped_ratio_numpy(ap0=ap_latent_dists_l1, an0=an_latent_dists_l1, ap1=ap_data_dists,      an1=an_data_dists),
-        'rsame_ground_latent.l2': _unswapped_ratio_numpy(ap0=ap_ground_dists,    an0=an_ground_dists,    ap1=ap_latent_dists_l2, an1=an_latent_dists_l2),  # simplifies to: (ap_latent_dists > an_latent_dists).to(torch.float32).mean()
-        'rsame_latent_data.l2':   _unswapped_ratio_numpy(ap0=ap_latent_dists_l2, an0=an_latent_dists_l2, ap1=ap_data_dists,      an1=an_data_dists),
-        # - compute the spearman rank correlation coefficient over the concatenated distances
-        'rcorr_ground_latent.l1': spearmanr(ground_dists,    latent_dists_l1)[0],
-        'rcorr_latent_data.l1':   spearmanr(latent_dists_l1, data_dists)[0],
-        'rcorr_ground_latent.l2': spearmanr(ground_dists,    latent_dists_l2)[0],
-        'rcorr_latent_data.l2':   spearmanr(latent_dists_l2, data_dists)[0],
-        # - compute the pearman correlation coefficient over the concatenated distances
-        'lcorr_ground_latent.l1': pearsonr(ground_dists, latent_dists_l1)[0],
-        'lcorr_latent_data.l1':   pearsonr(latent_dists_l1, data_dists)[0],
-        'lcorr_ground_latent.l2': pearsonr(ground_dists, latent_dists_l2)[0],
-        'lcorr_latent_data.l2':   pearsonr(latent_dists_l2, data_dists)[0],
-    })
+    scores.update(
+        {
+            # - check the number of swapped elements along a factor for random triplets.
+            "rsame_ground_latent.l1": _unswapped_ratio_numpy(
+                ap0=ap_ground_dists, an0=an_ground_dists, ap1=ap_latent_dists_l1, an1=an_latent_dists_l1
+            ),  # simplifies to: (ap_latent_dists > an_latent_dists).to(torch.float32).mean()
+            "rsame_latent_data.l1": _unswapped_ratio_numpy(
+                ap0=ap_latent_dists_l1, an0=an_latent_dists_l1, ap1=ap_data_dists, an1=an_data_dists
+            ),
+            "rsame_ground_latent.l2": _unswapped_ratio_numpy(
+                ap0=ap_ground_dists, an0=an_ground_dists, ap1=ap_latent_dists_l2, an1=an_latent_dists_l2
+            ),  # simplifies to: (ap_latent_dists > an_latent_dists).to(torch.float32).mean()
+            "rsame_latent_data.l2": _unswapped_ratio_numpy(
+                ap0=ap_latent_dists_l2, an0=an_latent_dists_l2, ap1=ap_data_dists, an1=an_data_dists
+            ),
+            # - compute the spearman rank correlation coefficient over the concatenated distances
+            "rcorr_ground_latent.l1": spearmanr(ground_dists, latent_dists_l1)[0],
+            "rcorr_latent_data.l1": spearmanr(latent_dists_l1, data_dists)[0],
+            "rcorr_ground_latent.l2": spearmanr(ground_dists, latent_dists_l2)[0],
+            "rcorr_latent_data.l2": spearmanr(latent_dists_l2, data_dists)[0],
+            # - compute the pearman correlation coefficient over the concatenated distances
+            "lcorr_ground_latent.l1": pearsonr(ground_dists, latent_dists_l1)[0],
+            "lcorr_latent_data.l1": pearsonr(latent_dists_l1, data_dists)[0],
+            "lcorr_ground_latent.l2": pearsonr(ground_dists, latent_dists_l2)[0],
+            "lcorr_latent_data.l2": pearsonr(latent_dists_l2, data_dists)[0],
+        }
+    )
 
     # [DONE]
     return scores
@@ -483,14 +538,14 @@ def _compute_scores_from_dists(dists: Dict[str, np.array]) -> Dict[str, float]:
 
 
 def _compute_factored_metric_components_along_factor(
-        dataset: DisentDataset,
-        representation_function,
-        f_idx: int,
-        num_samples: int,
-        repeats: int,
-        batch_size: int,
-        compute_distances: bool,
-        compute_linearity: bool,
+    dataset: DisentDataset,
+    representation_function,
+    f_idx: int,
+    num_samples: int,
+    repeats: int,
+    batch_size: int,
+    compute_distances: bool,
+    compute_linearity: bool,
 ) -> Dict[str, float]:
     # NOTE: what to do if the factor size is too small?
 
@@ -505,13 +560,17 @@ def _compute_factored_metric_components_along_factor(
         # [ENCODE TRAVERSAL]:
         # - generate repeated factors, varying one factor over the entire range
         # - shape: (factor_size, z_size)
-        zs_traversal, xs_traversal = encode_all_along_factor(dataset, representation_function, f_idx=f_idx, batch_size=batch_size, return_batch=True)
+        zs_traversal, xs_traversal = encode_all_along_factor(
+            dataset, representation_function, f_idx=f_idx, batch_size=batch_size, return_batch=True
+        )
         zs_traversal = zs_traversal.cpu()
 
         if compute_distances:
             xs_traversal = xs_traversal.cpu()
             # [COMPUTE SAME RATIO & CORRELATION] | was: `num_triplets=_SAMPLES_MULTIPLIER_FACTOR*len(zs_traversal)`
-            computed_dists = _compute_dists(num_triplets=num_samples, zs_traversal=zs_traversal, xs_traversal=xs_traversal, factors=None)
+            computed_dists = _compute_dists(
+                num_triplets=num_samples, zs_traversal=zs_traversal, xs_traversal=xs_traversal, factors=None
+            )
             # [STORE DISTANCES]
             distance_measures.append(computed_dists)
 
@@ -519,20 +578,22 @@ def _compute_factored_metric_components_along_factor(
             # [VARIANCE ALONG DIFFERING AXES]:
             # 1. axis: correlation with standard basis (1, 0, 0, ...), (0, 1, 0, ...), ...
             # 2. linear: correlation along arbitrary orthogonal bases
-            axis_values_var = _compute_unsorted_axis_values(zs_traversal, use_std=False)      # shape: (z_size,)
+            axis_values_var = _compute_unsorted_axis_values(zs_traversal, use_std=False)  # shape: (z_size,)
             linear_values_var = _compute_unsorted_linear_values(zs_traversal, use_std=False)  # shape: (z_size,)
             # [COMPUTE LINEARITY SCORES]:
-            axis_ratio_var = _score_from_unsorted(axis_values_var, top_2=False, norm=True)      # shape: ()
+            axis_ratio_var = _score_from_unsorted(axis_values_var, top_2=False, norm=True)  # shape: ()
             linear_ratio_var = _score_from_unsorted(linear_values_var, top_2=False, norm=True)  # shape: ()
             # [STORE SCORES]
-            linear_measures.append({
-                'linearity.axis_ratio.var': axis_ratio_var,
-                'linearity.linear_ratio.var': linear_ratio_var,
-                # aggregating linear values outside this function does not make sense, values do not correspond between repeats.
-                'linearity.axis_alignment.var': axis_ratio_var / (linear_ratio_var + 1e-20),
-                # temp values
-                '_TEMP_.axis_values.var': axis_values_var,
-            })
+            linear_measures.append(
+                {
+                    "linearity.axis_ratio.var": axis_ratio_var,
+                    "linearity.linear_ratio.var": linear_ratio_var,
+                    # aggregating linear values outside this function does not make sense, values do not correspond between repeats.
+                    "linearity.axis_alignment.var": axis_ratio_var / (linear_ratio_var + 1e-20),
+                    # temp values
+                    "_TEMP_.axis_values.var": axis_values_var,
+                }
+            )
 
     # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~- #
     # AGGREGATE DATA - For each distance measure
@@ -542,8 +603,8 @@ def _compute_factored_metric_components_along_factor(
         # concatenate all into arrays: <shape: (repeats*num_samples,)>
         # then aggregate over first dimension: <shape: (,)>
         distance_measures: Dict[str, np.ndarray] = _numpy_concat_all_dicts(distance_measures)
-        distance_measures: Dict[str, float]      = _compute_scores_from_dists(distance_measures)
-        distance_measures: Dict[str, float]      = {f'distances.{k}.factor': v for k, v in distance_measures.items()}
+        distance_measures: Dict[str, float] = _compute_scores_from_dists(distance_measures)
+        distance_measures: Dict[str, float] = {f"distances.{k}.factor": v for k, v in distance_measures.items()}
     else:
         distance_measures: Dict[str, float] = {}
 
@@ -559,7 +620,9 @@ def _compute_factored_metric_components_along_factor(
         linear_measures: Dict[str, torch.Tensor] = _torch_stack_all_dicts(linear_measures)
         linear_measures: Dict[str, torch.Tensor] = {k: v.mean(dim=0) for k, v in linear_measures.items()}
         # compute average scores & remove keys
-        linear_measures['linearity.axis_ratio_ave.var'] = _score_from_unsorted(linear_measures.pop('_TEMP_.axis_values.var'), top_2=False, norm=True)  # shape: (z_size,) -> ()
+        linear_measures["linearity.axis_ratio_ave.var"] = _score_from_unsorted(
+            linear_measures.pop("_TEMP_.axis_values.var"), top_2=False, norm=True
+        )  # shape: (z_size,) -> ()
         # convert values
         linear_measures: Dict[str, float] = {k: float(v) for k, v in linear_measures.items()}
     else:
