@@ -31,7 +31,6 @@ import torch
 
 from disent.nn.modules import DisentModule
 
-
 log = logging.getLogger(__name__)
 
 
@@ -47,13 +46,12 @@ log = logging.getLogger(__name__)
 
 
 class DisentLatentsModule(DisentModule):
-
     def __init__(self, x_shape=(3, 64, 64), z_size=6, z_multiplier=1):
         super().__init__()
-        self._x_shape = x_shape
+        self._x_shape = tuple(x_shape)
         self._x_size = int(np.prod(x_shape))
-        self._z_size = z_size
-        self._z_multiplier = z_multiplier
+        self._z_size = int(z_size)
+        self._z_multiplier = int(z_multiplier)
 
     def forward(self, *args, **kwargs):
         raise NotImplementedError
@@ -61,15 +59,19 @@ class DisentLatentsModule(DisentModule):
     @property
     def x_shape(self):
         return self._x_shape
+
     @property
     def x_size(self):
         return self._x_size
+
     @property
     def z_size(self):
         return self._z_size
+
     @property
     def z_multiplier(self):
         return self._z_multiplier
+
     @property
     def z_total(self):
         return self._z_size * self._z_multiplier
@@ -81,13 +83,12 @@ class DisentLatentsModule(DisentModule):
 
 
 class DisentEncoder(DisentLatentsModule):
-
     @final
     def forward(self, x, chunk=True) -> torch.Tensor:
         """same as self.encode but with size checks"""
         # checks
-        assert x.ndim == 4, f'ndim mismatch: 4 (required) != {x.ndim} (given)'
-        assert x.shape[1:] == self.x_shape, f'x_shape mismatch: {self.x_shape} (required) != {x.shape[1:]} (batch)'
+        assert x.ndim == 4, f"ndim mismatch: 4 (required) != {x.ndim} (given) [shape={x.shape}]"
+        assert x.shape[1:] == self.x_shape, f"x_shape mismatch: {self.x_shape} (required) != {x.shape[1:]} (batch)"
         # encode | p(z|x)
         # for a gaussian encoder, we treat z as concat(z_mean, z_logvar) where z_mean.shape == z_logvar.shape
         # ie. the first half of z is z_mean, the second half of z is z_logvar
@@ -104,9 +105,8 @@ class DisentEncoder(DisentLatentsModule):
 
 
 class DisentDecoder(DisentLatentsModule):
-
     def __init__(self, x_shape=(3, 64, 64), z_size=6, z_multiplier=1):
-        assert z_multiplier == 1, 'decoder does not support z_multiplier != 1'
+        assert z_multiplier == 1, "decoder does not support z_multiplier != 1"
         super().__init__(x_shape=x_shape, z_size=z_size, z_multiplier=z_multiplier)
 
     @final
@@ -131,14 +131,13 @@ class DisentDecoder(DisentLatentsModule):
 
 
 class AutoEncoder(DisentLatentsModule):
-
     def __init__(self, encoder: DisentEncoder, decoder: DisentDecoder):
         assert isinstance(encoder, DisentEncoder)
         assert isinstance(decoder, DisentDecoder)
         # check sizes
-        assert encoder.x_shape == decoder.x_shape, 'x_shape mismatch'
-        assert encoder.x_size == decoder.x_size, 'x_size mismatch - this should never happen if x_shape matches'
-        assert encoder.z_size == decoder.z_size, 'z_size mismatch'
+        assert encoder.x_shape == decoder.x_shape, "x_shape mismatch"
+        assert encoder.x_size == decoder.x_size, "x_size mismatch - this should never happen if x_shape matches"
+        assert encoder.z_size == decoder.z_size, "z_size mismatch"
         # initialise
         super().__init__(x_shape=decoder.x_shape, z_size=decoder.z_size, z_multiplier=encoder.z_multiplier)
         # assign
@@ -146,7 +145,7 @@ class AutoEncoder(DisentLatentsModule):
         self._decoder = decoder
 
     def forward(self, x):
-        raise RuntimeError(f'{self.__class__.__name__}.forward(...) has been disabled')
+        raise RuntimeError(f"{self.__class__.__name__}.forward(...) has been disabled")
 
     def encode(self, x, chunk=False):
         z_raw = self._encoder(x, chunk=chunk)
