@@ -30,24 +30,27 @@ import pytest
 from disent.util import _set_test_run
 from tests.util import no_stdout
 
-
 # ========================================================================= #
 # TESTS                                                                     #
 # ========================================================================= #
 
 
-ROOT_DIR = os.path.abspath(__file__ + '/../..')
+ROOT_DIR = os.path.abspath(__file__ + "/../..")
 
 
-@pytest.mark.parametrize("module", [
-    os.path.relpath(path, ROOT_DIR).replace('/', '.')[:-3]
-    for path in glob(os.path.join(ROOT_DIR, 'docs/examples/overview_**.py'))
-])
+@pytest.mark.parametrize(
+    "module",
+    [
+        os.path.relpath(path, ROOT_DIR).replace("/", ".")[:-3]
+        for path in glob(os.path.join(ROOT_DIR, "docs/examples/overview_**.py"))
+    ],
+)
 def test_docs_examples(capsys, module):
     # make sure everything doesnt take 5 years to run
     _set_test_run()
     # run all the files in the examples folder
     import importlib
+
     with no_stdout():
         importlib.import_module(module)
 
